/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.taxonomy.factsheet.spi;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.io.File;
import java.io.InputStream;
import java.io.IOException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.openrdf.model.Graph;
import it.tidalwave.util.As;
import it.tidalwave.util.Id;
import it.tidalwave.util.thread.ThreadType;
import it.tidalwave.util.thread.annotation.ThreadConfined;
import it.tidalwave.role.ui.PresentationModel;
import it.tidalwave.role.ui.spi.DefaultPresentationModel;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.observation.ObservationSet;
import it.tidalwave.mobile.asset.SmartAssetManager;
import it.tidalwave.semantic.io.GraphDeserializer;
import it.tidalwave.semantic.io.GraphUnmarshaller;
import it.tidalwave.semantic.io.MimeTypes;
import it.tidalwave.semantic.io.impl.DefaultGraphDeserializer;
import it.tidalwave.semantic.io.impl.GraphUnmarshallerImpl;
import it.tidalwave.bluebill.taxonomy.mobile.Taxon;
import it.tidalwave.bluebill.mobile.role.util.RoleRegister;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.TaxonFactSheetView;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.TaxonFactSheetViewController;
import it.tidalwave.util.AsException;
import lombok.Cleanup;
import lombok.RequiredArgsConstructor;
import static org.openide.util.NbBundle.*;
import static it.tidalwave.role.Composite.Composite;
import static it.tidalwave.role.Identifiable.Identifiable;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@RequiredArgsConstructor
public abstract class TaxonFactSheetViewControllerSupport<View extends TaxonFactSheetView> 
                                                               implements TaxonFactSheetViewController<View>
  {
    private static final Logger log = LoggerFactory.getLogger(TaxonFactSheetViewControllerSupport.class);
    
    private static final Class<TaxonFactSheetViewControllerSupport> _ = TaxonFactSheetViewControllerSupport.class;
    
    private static final Id TYPE_OBSERVATION_SET = new Id("http://www.tidalwave.it/rdf/observation/2010/07/01#ObservationSet"); // FIXME: put in vocabulary
    
    @Nonnull
    protected final View view;

    private final DefaultPresentationModel presentationModel = new DefaultPresentationModel(); // FIXME: PresentationModel

    protected final RoleRegister roleRegister = new RoleRegister();
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public PresentationModel getPresentationModel (final @Nonnegative int index)
      {
        return presentationModel.getChild(index);
      }

    /************************************************h*******************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.ANY)
    public final void initialize (final @Nonnull Taxon taxon)
      {
        log.info("initialize()");
        
        view.showPleaseWaitAndLockUI(getMessage(_, "preparingData"));
        
        new Thread()
          {
            @Override
            public void run()
              {
                try 
                  {
                    clearPresentationModels();
                    createPresentationModels(taxon);
                  }
                catch (Throwable e)
                  {
                    log.error("While creating PresentationModels: {}", e.toString());
                    log.error("", e);
                    // FIXME: notify error on View, stop progress dialog
                  }
                    
                populate();
              }
          }.start();
      }    
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void openWebPage (final @Nonnull As entity) // FIXME: use PresentationModel
      {
        try
          {
            view.openWebPage(entity.as(Identifiable).getId().stringValue());
          }
        catch (Exception e)
          {
            log.warn("Can't open web page {}", e);
            log.warn("", e);
            // FIXME: non fatal notification to the view
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.NOT_UI)
    protected void clearPresentationModels()
      {
        presentationModel.clear();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.NOT_UI)
    protected void addPresentationModel (final @Nonnull PresentationModel presentationModel)
      {
        this.presentationModel.addChild(presentationModel);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.NOT_UI)
    protected abstract void createPresentationModels (final @Nonnull Taxon taxon)
      throws Exception;
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected void populate() // FIXME rename to populateView
      {
        log.info("populate() - presentation model: {}", presentationModel);

        try
          {
            if (presentationModel.as(Composite).findChildren().count() > 0)
              {
                view.populate(presentationModel);            
              }
            else
              {
                view.notifyNoData(getMessage(_, "noData"));  
              }
          }
        catch (AsException e)
          {
            view.notifyNoData(getMessage(_, "noData"));  
          }
        
        view.dismissPleaseWaitAndUnlockUI();
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public ObservationSet loadObservationSet (final @Nonnull Taxon taxon) // FIXME: should be protected, 
      throws IOException
      {
        // FIXME: now it loads xeno-canto, but should be a generic observation set with any kind of media
        log.info("loadObservationSet()");
        final String resourceName = "xenocanto-" + taxon.getId().stringValue().replaceAll(":", "_") + ".n3";
        log.info(">>>> resourceName: {}", resourceName);

        final File file = Locator.find(SmartAssetManager.class).findAsset("xeno-canto.zip");
        log.info(">>>> zip file: {}", file);
        final @Cleanup ZipFile zipFile = new ZipFile(file);
        final ZipEntry entry = zipFile.getEntry(resourceName);

        if (entry == null)
          {
            throw new IOException("No entry for " + resourceName);
          }

        final @Cleanup InputStream is = zipFile.getInputStream(entry);
        return loadObservationSet(is, MimeTypes.MIME_N3);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected static ObservationSet loadObservationSet (final @Nonnull InputStream is, final @Nonnull String mimeType)
      throws IOException
      {
        try
          {
            final GraphDeserializer deserializer = new DefaultGraphDeserializer();
            final Graph graph = deserializer.read(is, mimeType);
            final GraphUnmarshaller unmarshaller = new GraphUnmarshallerImpl();
            return (ObservationSet)unmarshaller.unmarshal(graph, TYPE_OBSERVATION_SET);
          }
        catch (Exception e)
          {
            throw new RuntimeException(e);
//            throw new IOException(e.toString()); // Java 5 / Android compatibility
          }
      }    
  }